@@ -5,8 +5,9 @@ package Devel::Optic;
 
 use Carp qw(croak);
 use Scalar::Util qw(looks_like_number);
-use Ref::Util qw(is_arrayref is_hashref is_scalarref is_refref);
+use Ref::Util qw(is_ref is_arrayref is_hashref is_scalarref is_coderef is_regexpref);
 
+use Sub::Info qw(sub_info);
 use Devel::Size qw(total_size);
 use PadWalker qw(peek_my);
 
@@ -22,7 +23,7 @@ use constant {
 
     DEFAULT_SCALAR_TRUNCATION_SIZE => 256,
     DEFAULT_SCALAR_SAMPLE_SIZE => 64,
-    DEFAULT_REF_KEY_SAMPLE_COUNT => 4,
+    DEFAULT_SAMPLE_COUNT => 4,
 };
 
 sub new {
@@ -49,7 +50,7 @@ sub new {
 
         # how many keys or indicies to display in a sample from an over-size
         # hashref/arrayref
-        ref_key_sample_count => $params{ref_key_sample_count} // DEFAULT_REF_KEY_SAMPLE_COUNT,
+        sample_count => $params{sample_count} // DEFAULT_SAMPLE_COUNT,
 
         lens => $params{lens} // Devel::Optic::Lens::Perlish->new,
     };
@@ -78,32 +79,49 @@ sub fit_to_view {
         return $subject;
     }
 
+    my $ref = ref $subject;
+    my $reasonably_summarized_with_substr = !is_ref($subject) || is_regexpref($subject) || is_scalarref($subject);
+
     # now we're in too-big territory, so we need to come up with a way to get
     # some useful data to the user without showing the whole structure
-    my $ref = ref $subject;
-    if (!$ref) {
+    if ($reasonably_summarized_with_substr) {
+        $subject = $$subject if is_scalarref($subject);
         my $scalar_truncation_size = $self->{scalar_truncation_size};
+        my $len = length $subject;
+
         # simple scalars we can truncate (PadWalker always returns refs, so
         # this is pretty safe from accidentally substr-ing an array or hash).
         # Also, once we know we're dealing with a gigantic string (or
         # number...), we can trim much more aggressively without hurting user
         # understanding too much.
+
+        if ($len <= $scalar_truncation_size) {
+            return sprintf(
+                "%s%s (len %d / %d bytes)",
+                $ref ? "$ref " : "",
+                $subject,
+                $len, $size,
+            );
+        }
+
         return sprintf(
-            "%s (truncated to length %d; length %d / %d bytes in full)",
-            substr($subject, 0, $scalar_truncation_size),
+            "%s%s (truncated to len %d; len %d / %d bytes in full)",
+            $ref ? "$ref " : "",
+            substr($subject, 0, $scalar_truncation_size) . "...",
             $scalar_truncation_size,
-            length $subject,
-            $size
+            $len, $size
         );
     }
 
-    my $ref_key_sample_count = $self->{ref_key_sample_count};
+    my $sample_count = $self->{sample_count};
     my $scalar_sample_size = $self->{scalar_sample_size};
-    my $sample_text = "No sample for type '$ref'";
+    my $sample_text = "$size bytes";
     if (is_hashref($subject)) {
         my @sample;
         my @keys = keys %$subject;
-        my @sample_keys = @keys[0 .. $ref_key_sample_count - 1];
+        my $key_count = scalar @keys;
+        $sample_count = $key_count > $sample_count ? $sample_count : $key_count;
+        my @sample_keys = @keys[0 .. $sample_count - 1];
         for my $key (@sample_keys) {
             my $val = $subject->{$key};
             my $val_chunk;
@@ -117,12 +135,16 @@ sub fit_to_view {
             $key_chunk .= '...' if length($key_chunk) < length($key);
             push @sample, sprintf("%s => %s", $key_chunk, $val_chunk);
         }
-        $sample_text = sprintf("{%s ...} (%d keys / %d bytes)", join(', ', @sample), scalar @keys, $size);
+        $sample_text = sprintf("{%s%s} (%d keys / %d bytes)",
+            join(', ', @sample),
+            $key_count > $sample_count ? ' ...' : '',
+            $key_count, $size
+        );
     } elsif (is_arrayref($subject)) {
         my @sample;
         my $total_len = scalar @$subject;
-        my $sample_len = $total_len > $ref_key_sample_count ? $ref_key_sample_count : $total_len;
-        for (my $i = 0; $i < $sample_len; $i++) {
+        $sample_count = $total_len > $sample_count ? $sample_count : $total_len;
+        for (my $i = 0; $i < $sample_count; $i++) {
             my $val = $subject->[$i];
             my $val_chunk;
             if (ref $val) {
@@ -133,10 +155,23 @@ sub fit_to_view {
             }
             push @sample, $val_chunk;
         }
-        $sample_text = sprintf("[%s ...] (len %d / %d bytes)", join(', ', @sample), $total_len, $size);
+        $sample_text = sprintf("[%s%s] (len %d / %d bytes)",
+            join(', ', @sample),
+            $total_len > $sample_count ? ' ...' : '',
+            $total_len, $size
+        );
+    } elsif (is_coderef($subject)) {
+        my $info = sub_info($subject);
+        $sample_text = sprintf("sub %s { ... } (L%d-%d in %s (%s))",
+            $info->{name},
+            $info->{start_line},
+            $info->{end_line},
+            $info->{package},
+            $info->{file},
+        );
     }
 
-    return sprintf("$ref: $sample_text. Exceeds viewing size (%d bytes)", $max_size);
+    return "$ref: $sample_text";
 }
 
 1;
@@ -153,7 +188,7 @@ Devel::Optic - Production safe data inspector
 
 =head1 VERSION
 
-version 0.007
+version 0.008
 
 =head1 SYNOPSIS
 
@@ -234,7 +269,7 @@ viewing. Default: 256.
 Size, in C<substr> length terms, that scalar children of a summarized data
 structure are trimmed to for inclusion in the summary. Default: 64.
 
-=item C<ref_key_sample_count>
+=item C<sample_count>
 
 Number of keys/indices to display when summarizing a hash or arrayref. Default: 4.
 
